/**
 * Created by Chreem on 2017/4/10.
 */
'use strict';

//------------------------------↓公用函数↓-----------------------------
function checkArray(ary) {
    // TODO 更全面的判断
    return ary.constructor === Array;
}

function checkFunction(func) {
    // TODO 更全面的判断
    return typeof func === 'function';
}

function extend(obj1, obj2) {
    for (var key in obj2) {
        obj1[key] && typeof obj1[key] === 'object'
            ? extend(obj1[key], obj2[key])
            : obj1[key] = obj2[key];
    }
    return obj1;
}
//------------------------------↑公用函数↑-----------------------------


/**
 * vue生命周期hooks
 * @param {VueInstance} instance VueInstance实例本身
 * @param {Vue} vm vue实例
 */
function beforeCreate(instance, vm) { checkArray ? instance._beforeCreate.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function created(instance, vm) { checkArray ? instance._created.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function beforeMount(instance, vm) { checkArray ? instance._beforeMount.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function mounted(instance, vm) { checkArray ? instance._mounted.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function beforeUpdate(instance, vm) { checkArray ? instance._beforeUpdate.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function updated(instance, vm) { checkArray ? instance._updated.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function beforeDestroy(instance, vm) { checkArray ? instance._beforeDestroy.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }
function destroy(instance, vm) { checkArray ? instance._destroy.map(function (func) { checkFunction(func) ? func.bind(vm)(vm) : void 0; }) : void 0; }

// TODO 是否有更优雅的方式来处理这8个生命周期
VueInstance.prototype.beforeCreate = function (func) { this._beforeCreate.push(func); };
VueInstance.prototype.created = function (func) { this._created.push(func); };
VueInstance.prototype.beforeMount = function (func) { this._beforeMount.push(func); };
VueInstance.prototype.mounted = function (func) { this._mounted.push(func); };
VueInstance.prototype.beforeUpdate = function (func) { this._beforeUpdate.push(func); };
VueInstance.prototype.updated = function (func) { this._updated.push(func); };
VueInstance.prototype.beforeDestroy = function (func) { this._beforeDestroy.push(func); };
VueInstance.prototype.destroy = function (func) { this._destroy.push(func); };


/**
 * 将传入对象深拷贝至当前data下
 */
VueInstance.prototype.data = function (obj) {
    extend(this._data, obj);
};

/**
 * 接受的参数同vue:
 * 传入string,array则可选类型默认null(accept any)
 * 传入obj则合并
 */
VueInstance.prototype.props = function (element) {
    if (typeof element === 'string') {
        var result = {};
        result[element] = null;
        extend(this._props, result);
    }
    if (typeof element === 'object') {
        var result = {};
        if (checkArray(element)) {
            element.forEach(function (item) { result[item] = null; });
        } else { result = element; }
        extend(this._props, result);
    }
};

/**
 * 只有传入obj才和computed合并
 */
VueInstance.prototype.computed = function (obj) {
    if (obj instanceof Object) {
        if (checkArray(obj)) { return; }
        if (checkFunction(obj)) { return; }
        // TODO 函数内部使用vue实例
        extend(this._computed, obj);
    }
};

/**
 * 传入obj才和methods合并
 */
VueInstance.prototype.methods = function (func) {
    if (func instanceof Object) {
        if (checkArray(func)) { return; }
        if (checkFunction(func)) { return; }
        // TODO 函数内部使用vue实例
        extend(this._methods, func);
    }
};

// methods

function VueInstance() {
    // vue生命周期执行的各方法
    this._beforeCreate = [];
    this._created = [];
    this._beforeMount = [];
    this._mounted = [];
    this._beforeUpdate = [];
    this._updated = [];
    this._beforeDestroy = [];
    this._destroy = [];

    this._props = {};
    this._data = {};
    this._computed = {};
    this._methods = {};

    this.vm = (function (instance) {
        /**
         * vue实例
         */
        return {
            beforeCreate: function () { beforeCreate(instance, this); },
            created: function () { created(instance, this); },
            beforeMount: function () { beforeMount(instance, this); },
            mounted: function () { mounted(instance, this); },
            beforeUpdate: function () { beforeUpdate(instance, this); },
            updated: function () { updated(instance, this); },
            beforeDestroy: function () { beforeDestroy(instance, this); },
            destroy: function () { destroy(instance, this); },

            props: instance._props,
            data: function () { return instance._data; },
            computed: instance._computed,
            methods: instance._methods
        }
    })(this);
}


/**
 * How to use:
 * 
 * // xxx.vue
 * <script>
 * import VueInstance from 'vue-instance';
 * ...
 * export default VueInstance.getInstance();
 * </script>
 * 
 * 总觉着这种方式返回不够炫...然而author又太菜
 */
VueInstance.prototype.getInstance = function () {
    // TODO 返回方式优化
    return this.vm;
};

module.exports = VueInstance;
